@@ -1,20 +1,22 @@
-/*
- *  Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
+/* This file is part of Spectacle, the KDE screenshot utility
+ * Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ *
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ExportManager.h"
@@ -29,12 +31,11 @@
 #include <QClipboard>
 #include <QPainter>
 #include <QFileDialog>
-#include <QBuffer>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
+#include <QString>
 
 #include <KSharedConfig>
-#include <KConfigGroup>
 #include <KIO/ListJob>
 #include <KIO/MkpathJob>
 #include <KIO/FileCopyJob>
@@ -48,7 +49,7 @@ ExportManager::ExportManager(QObject *parent) :
     mTempFile(QUrl()),
     mTempDir(nullptr)
 {
-    connect(this, &ExportManager::imageSaved, [this](const QUrl &savedAt) {
+    connect(this, &ExportManager::imageSaved, [](const QUrl &savedAt) {
         SpectacleConfig::instance()->setLastSaveFile(savedAt);
     });
 }
@@ -81,14 +82,14 @@ QString ExportManager::windowTitle() const
     return mWindowTitle;
 }
 
-ImageGrabber::GrabMode ExportManager::grabMode() const
+Spectacle::CaptureMode ExportManager::captureMode() const
 {
-    return mGrabMode;
+    return mCaptureMode;
 }
 
-void ExportManager::setGrabMode(const ImageGrabber::GrabMode &grabMode)
+void ExportManager::setCaptureMode(const Spectacle::CaptureMode &theCaptureMode)
 {
-    mGrabMode = grabMode;
+    mCaptureMode = theCaptureMode;
 }
 
 void ExportManager::setPixmap(const QPixmap &pixmap)
@@ -109,20 +110,21 @@ void ExportManager::updatePixmapTimestamp()
 	mPixmapTimestamp = QDateTime::currentDateTime();
 }
 
+void ExportManager::setTimestamp(const QDateTime &timestamp)
+{
+    mPixmapTimestamp = timestamp;
+}
+
 // native file save helpers
 
 QString ExportManager::defaultSaveLocation() const
 {
-    QString savePath = SpectacleConfig::instance()->defaultSaveLocation();
-    if (savePath.isEmpty() || savePath.isNull()) {
-        savePath = QDir::homePath();
-    }
+    QString savePath = SpectacleConfig::instance()->defaultSaveLocation().toLocalFile();
     savePath = QDir::cleanPath(savePath);
 
     QDir savePathDir(savePath);
     if (!(savePathDir.exists())) {
         savePathDir.mkpath(QStringLiteral("."));
-        SpectacleConfig::instance()->setDefaultSaveLocation(savePath);
     }
 
     return savePath;
@@ -168,9 +170,9 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     const QString baseDir = defaultSaveLocation();
     QString title;
 
-    if (mGrabMode == ImageGrabber::GrabMode::ActiveWindow ||
-        mGrabMode == ImageGrabber::GrabMode::TransientWithParent ||
-        mGrabMode == ImageGrabber::GrabMode::WindowUnderCursor) {
+    if (mCaptureMode == Spectacle::CaptureMode::ActiveWindow ||
+        mCaptureMode == Spectacle::CaptureMode::TransientWithParent ||
+        mCaptureMode == Spectacle::CaptureMode::WindowUnderCursor) {
         title = mWindowTitle.replace(QLatin1String("/"), QLatin1String("_"));  // POSIX doesn't allow "/" in filenames
     } else {
         // Remove '%T' with separators around it
@@ -218,7 +220,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
             // ... check the file names for string matching token with padding specified in result
             const QStringList filteredFiles = fileNames.filter(fileNumberRE);
 
-            // if there are files in the direcory that look like the file name with sequential numbering
+            // if there are files in the directory that look like the file name with sequential numbering
             if (filteredFiles.length() > 0) {
                 // loop through filtered file names looking for highest number
                 for (const QString &filteredFile: filteredFiles) {
@@ -467,7 +469,8 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
     SpectacleConfig *config = SpectacleConfig::instance();
 
     // construct the supported mimetype list
-    Q_FOREACH (auto mimeType, QImageWriter::supportedMimeTypes()) {
+    const auto mimeTypes = QImageWriter::supportedMimeTypes();
+    for (const auto &mimeType : mimeTypes) {
         supportedFilters.append(QString::fromUtf8(mimeType).trimmed());
     }
 
@@ -485,7 +488,7 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 
     // launch the dialog
     if (dialog.exec() == QFileDialog::Accepted) {
-        const QUrl saveUrl = dialog.selectedUrls().first();
+        const QUrl saveUrl = dialog.selectedUrls().constFirst();
         if (saveUrl.isValid()) {
             if (save(saveUrl)) {
                 emit imageSaved(saveUrl);
@@ -503,12 +506,16 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 
 // misc helpers
 
-void ExportManager::doCopyToClipboard()
+void ExportManager::doCopyToClipboard(bool notify)
 {
     auto data = new QMimeData();
     data->setImageData(mSavePixmap.toImage());
     data->setData(QStringLiteral("x-kde-force-image-copy"), QByteArray());
     QApplication::clipboard()->setMimeData(data, QClipboard::Clipboard);
+
+    if (notify) {
+        emit forceNotify(QUrl());
+    }
 }
 
 void ExportManager::doPrint(QPrinter *printer)
