@@ -1,5 +1,4 @@
 /*
- *  Copyright (C) 2016 Boudhayan Gupta <bgupta@kde.org>
  *  Copyright (C) 2018 Ambareesh "Amby" Balaji <ambareeshbalaji@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,6 +18,8 @@
  */
 
 #include <KLocalizedString>
+#include <QGuiApplication>
+#include <QScreen>
 
 #include "QuickEditor.h"
 #include "SpectacleConfig.h"
@@ -45,7 +46,8 @@ const int QuickEditor::magZoom = 5;
 const int QuickEditor::magPixels = 16;
 const int QuickEditor::magOffset = 32;
 
-QuickEditor::QuickEditor(const QPixmap& pixmap) :
+QuickEditor::QuickEditor(const QPixmap& thePixmap, QWidget *parent) :
+    QWidget(parent),
     mMaskColor(QColor::fromRgbF(0, 0, 0, 0.15)),
     mStrokeColor(palette().highlight().color()),
     mCrossColor(QColor::fromRgbF(mStrokeColor.redF(), mStrokeColor.greenF(), mStrokeColor.blueF(), 0.7)),
@@ -61,7 +63,7 @@ QuickEditor::QuickEditor(const QPixmap& pixmap) :
     mBottomHelpTextFont(font()),
     mBottomHelpGridLeftWidth(0),
     mMouseDragState(MouseState::None),
-    mPixmap(pixmap),
+    mPixmap(thePixmap),
     mMagnifierAllowed(false),
     mShowMagnifier(SpectacleConfig::instance()->showMagnifierChecked()),
     mToggleMagnifier(false),
@@ -79,7 +81,6 @@ QuickEditor::QuickEditor(const QPixmap& pixmap) :
     setMouseTracking(true);
     setAttribute(Qt::WA_StaticContents);
     setWindowFlags(Qt::FramelessWindowHint | Qt::NoDropShadowWindowHint | Qt::Popup | Qt::WindowStaysOnTopHint);
-    show();
 
     dprI = 1.0 / devicePixelRatioF();
     setGeometry(0, 0, static_cast<int>(mPixmap.width() * dprI), static_cast<int>(mPixmap.height() * dprI));
@@ -109,7 +110,7 @@ QuickEditor::QuickEditor(const QPixmap& pixmap) :
         };
         for (auto& pair : mBottomHelpText) {
             prepare(pair.first);
-            for (auto item : pair.second) {
+            for (auto &item : pair.second) {
                 prepare(item);
             }
         }
@@ -287,6 +288,12 @@ int QuickEditor::boundsDown(int newTopLeftY, const bool mouse)
 void QuickEditor::mousePressEvent(QMouseEvent* event)
 {
     if (event->button() & Qt::LeftButton) {
+        /* NOTE  Workaround for Bug 407843  
+        * If we show the selection Widget when a right click menu is open we lose focus on X.
+        * When the user clicks we get the mouse back. We can only grab the keyboard if we already
+        * have mouse focus. So just grab it undconditionally here.
+        */
+        grabKeyboard();
         const QPointF& pos = event->pos();
         mMousePos = pos;
         mMagnifierAllowed = true;
@@ -430,7 +437,9 @@ void QuickEditor::mouseReleaseEvent(QMouseEvent* event)
             setCursor(Qt::OpenHandCursor);
         }
         else if(mMouseDragState == MouseState::Outside && mReleaseToCapture) {
-            acceptSelection();
+            event->accept();
+            mMouseDragState = MouseState::None;
+            return acceptSelection();
         }
     } else if (button == Qt::RightButton) {
         mSelection.setWidth(0);
