@@ -19,14 +19,53 @@
 
 #include "SpectacleConfig.h"
 
+#include <KLocalizedString>
 #include <KWindowSystem>
 
+
+
 SpectacleConfig::SpectacleConfig(QObject *parent) :
     QObject(parent)
 {
     mConfig = KSharedConfig::openConfig(QStringLiteral("spectaclerc"));
     mGeneralConfig = KConfigGroup(mConfig, "General");
     mGuiConfig = KConfigGroup(mConfig, "GuiConfig");
+
+    shortCutActions = new KActionCollection(this);
+
+    //everything here is named to match the jumplist actions in our .desktop file
+    shortCutActions->setComponentName(QStringLiteral("org.kde.spectacle.desktop"));
+    //qdbus org.kde.kglobalaccel /component/org_kde_spectacle_desktop org.kde.kglobalaccel.Component.shortcutNames
+    // ActiveWindowScreenShot
+    // CurrentMonitorScreenShot
+    // RectangularRegionScreenShot
+    // FullScreenScreenShot
+    // _launch
+    {
+        QAction *action = new QAction(i18n("Launch Spectacle"));
+        action->setObjectName(QStringLiteral("_launch"));
+        shortCutActions->addAction(action->objectName(), action);
+    }
+    {
+        QAction *action = new QAction(i18n("Capture Entire Desktop"));
+        action->setObjectName(QStringLiteral("FullScreenScreenShot"));
+        shortCutActions->addAction(action->objectName(), action);
+    }
+    {
+        QAction *action = new QAction(i18n("Capture Current Monitor"));
+        action->setObjectName(QStringLiteral("CurrentMonitorScreenShot"));
+        shortCutActions->addAction(action->objectName(), action);
+    }
+    {
+        QAction *action = new QAction(i18n("Capture Active Window"));
+        action->setObjectName(QStringLiteral("ActiveWindowScreenShot"));
+        shortCutActions->addAction(action->objectName(), action);
+    }
+    {
+        QAction *action = new QAction(i18n("Capture Rectangular Region"));
+        action->setObjectName(QStringLiteral("RectangularRegionScreenShot"));
+        shortCutActions->addAction(action->objectName(), action);
+    }
 }
 
 SpectacleConfig::~SpectacleConfig()
@@ -53,8 +92,7 @@ QString SpectacleConfig::defaultTimestampTemplate() const
 
 QUrl SpectacleConfig::lastSaveAsFile() const
 {
-    return mGeneralConfig.readEntry(QStringLiteral("lastSaveAsFile"),
-                                    QUrl(this->defaultSaveLocation()));
+    return mGeneralConfig.readEntry(QStringLiteral("lastSaveAsFile"), this->defaultSaveLocation());
 }
 
 void SpectacleConfig::setLastSaveAsFile(const QUrl &location)
@@ -72,8 +110,7 @@ QUrl SpectacleConfig::lastSaveAsLocation() const
 
 QUrl SpectacleConfig::lastSaveFile() const
 {
-    return mGeneralConfig.readEntry(QStringLiteral("lastSaveFile"),
-                                    QUrl(this->defaultSaveLocation()));
+    return mGeneralConfig.readEntry(QStringLiteral("lastSaveFile"), this->defaultSaveLocation());
 }
 
 void SpectacleConfig::setLastSaveFile(const QUrl &location)
@@ -260,7 +297,7 @@ void SpectacleConfig::setUseLightRegionMaskColour(bool enabled)
 
 int SpectacleConfig::compressionQuality() const
 {
-    return mGuiConfig.readEntry(QStringLiteral("compressionQuality"), 89);
+    return mGuiConfig.readEntry(QStringLiteral("compressionQuality"), 90);
 }
 
 void SpectacleConfig::setCompressionQuality(int value)
@@ -305,15 +342,19 @@ void SpectacleConfig::setAutoSaveFilenameFormat(const QString &format)
 
 // autosave location
 
-QString SpectacleConfig::defaultSaveLocation() const
+QUrl SpectacleConfig::defaultSaveLocation() const
 {
-    return mGeneralConfig.readPathEntry(QStringLiteral("default-save-location"),
-                          QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+    QString path = mGeneralConfig.readPathEntry(QStringLiteral("default-save-location"),
+        (QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)));
+    if (! path.endsWith(QLatin1Char('/'))) {
+        path += QLatin1Char('/');
+    }
+    return QUrl::fromUserInput(path);
 }
 
-void SpectacleConfig::setDefaultSaveLocation(const QString &location)
+void SpectacleConfig::setDefaultSaveLocation(const QUrl &location)
 {
-    mGeneralConfig.writePathEntry(QStringLiteral("default-save-location"), location);
+    mGeneralConfig.writePathEntry(QStringLiteral("default-save-location"), location.toString());
     mGeneralConfig.sync();
 }
 
